#include "lpc2103_keil.h"
#include "VIClowlevel.h"
#include "interrupts.h"
#include "led.h"
#include "spi.h"

#define PCLK_10MHz	1
#define PCLK ((unsigned long)10000000)
#define CCLK ((unsigned long)10000000)
#define TIMER_RATE 10

void initSystem(void);
void my_delay(void);

void timer_init(void)
{
	T0TCR = 0x02;				// disable timer, put in reset mode.
	T0CTCR = 0x00;				// select timer mode
	T0PR = 1000; 				// Set interrupt to fire at 2 Hz
	T0MR0 = PCLK / (TIMER_RATE * T0PR);	
	T0MCR = 0x0003;				// set an interrupt and reset when timer0 matches MR0 [man193]
	VICIntSelect = 0;			// assign all interrupts to IRQ category [man50]
	VICVectAddr0 = (unsigned long)timer_ISR;	// set interrupt vector address [man52]
	VICVectCntl0 = 0x00000024;	// interrupt source with index 4 (timer0) is enabled
								// as the one with priority 0. man[51], man[58]
	VICIntEnable = (1 << 4);	// enable interrupt for timer0 [man47]
	T0TCR = 0x01;				// take the timer out of reset mode and enable for counting.
}


  /**
  *  Configure the PLL and the MAM.
  *
  */
void initSystem(void)
{
	VICProtection = 0;						// Enabled user-mode code to change VIC state
	
	// PLL Setup
	#ifdef PCLK_10MHz
		PLLCFG = (1<<5)	| (1<<6);				// 10MHz
	#endif
	#ifdef PCLK_20MHz
		PLLCFG = (1<<0) | (1<<5) | (1<<6);		// 20MHz
	#endif
	#ifdef PCLK_30MHz
		PLLCFG = (1<<1)	| (1<<6);				// 30MHz
	#endif
	#ifdef PCLK_40MHz
		PLLCFG = (1<<0) | (1<<1) | (1<<6);		// 40MHz
	#endif
	#ifdef PCLK_50MHz
		PLLCFG = (1<<2) | (1<<5);				// 50MHz
	#endif
	#ifdef PCLK_60MHz
		PLLCFG = (1<<0) | (1<<2) | (1<<5);		// 60MHz
	#endif
	#ifdef PCLK_70MHz
		PLLCFG = (1<<1) | (1<<2) | (1<<5);		// 70MHz
	#endif
	
	PLLFEED = 0xAA;
	PLLFEED = 0x55;	
	PLLCON  = (1<<0);						// Enable PLL
	PLLFEED = 0xAA;
	PLLFEED = 0x55;	
	while(!(PLLSTAT & (1<<10)));			// Wait for PLL to be locked
	PLLCON = (1<<1) | (1<<0);				// Connect PLL
	PLLFEED = 0xAA;
	PLLFEED = 0x55;              
	
	APBDIV = 1; 							// Peripheral clock = processor clock

	SCS |= (1<<0);							// Enable fast IO pins

	MEMMAP = 0x01;							// Use user-mode flash memory mapping

	// Memory acceleration
	MAMCR = 0;								// Start change by turning off the MAM
	MAMTIM = 3;  							// Set MAM-Fetch cycle to 3 cclk as recommended for >40MHz
	MAMCR = 2; 								// Enable MAM
}


void LED1_init(void)
{
	LED1_PINSEL  =  ( LED1_PINSEL & ~(3<<LED1_PS_BIT) );
	FIODIR      |=  (1 << LED1_BIT);
	FIOMASK     &= ~(1 << LED1_BIT);
	FIOSET      |=  (1 << LED1_BIT);
}

// delay for about 0.5 secs
void my_delay(void)
{
	volatile unsigned long i;
	for(i = 0; i < 1000000; i++);
}

void pulse_led(void)
{
	FIOCLR |= (1 << LED1_BIT);
	my_delay();
	FIOSET |= (1 << LED1_BIT);
	my_delay();
}

int main(void)
{
	// Set PLL and MAM
	initSystem();
	
	// Enable SPI
	initSPI1(16);		
	
	LED1_init();
	pulse_led();		// pulse the LED to show we're alive
	timer_init();		// configure the timer
	enableIRQ();		// enable interrupts
	
	
	while(1);	// loop forever
	
	return 0;
}

